import pickle

from tf_core.document_corpus import DocumentCorpus
from tf_core.nltoolkit.library import corpus_reader
from nltk.tag.sequential import (DefaultTagger, NgramTagger, AffixTagger,
                                 RegexpTagger, #<--TODO
                                 ClassifierBasedPOSTagger)
import nltk.tag.brill
from nltk.tag.brill      import BrillTagger
from nltk.tag.brill_trainer import BrillTaggerTrainer
from nltk.tag.tnt        import TnT
from nltk.tag.hunpos     import HunposTagger
from nltk.tag.stanford   import StanfordTagger
#from nltk.tag.crf        import MalletCRF
from django.conf import settings
from nltk.corpus import brown, treebank, nps_chat
from nltk.tag.stanford import StanfordPOSTagger
import os
import re
import subprocess
from tree_tagger import TreeTagger
from crf_tagger import CRFTagger, PickleableCRFTagger
from nltk.tag import tnt
import nlp4j


class StanfordTagger(StanfordPOSTagger):

    def __init__(self, *args, **kwargs):
        super(StanfordTagger, self).__init__(*args, **kwargs)

    def train(self, corpus, widget_id):
        output_path = os.path.join(settings.STANFORD_POS_TAGGER, str(widget_id) + '.tsv')
        output_model = os.path.join(settings.STANFORD_POS_TAGGER,  str(widget_id) + '.tagger')
        if os.path.exists(output_model):
            os.remove(output_model)
        if os.path.exists(output_path):
            os.remove(output_path)
        with open(output_path, 'a') as f:
            for sentence in corpus:
                s = []
                for word, tag in sentence:
                    if word.strip() and tag.strip():
                        token = word.strip().replace(" ", "") + '_' + tag.strip()
                        if '_' in token:
                            s.append(token)
                new_sentence = " ".join(s)
                f.write(" " + new_sentence + " ")
        f.close()
        subprocess.call(
            ['java', '-mx4g',
             '-classpath', settings.STANFORD_POS_TAGGER_JAR,
             'edu.stanford.nlp.tagger.maxent.MaxentTagger',
             '-model', os.path.join(settings.STANFORD_POS_TAGGER,  str(widget_id) + '.tagger'),
             '-trainFile', os.path.join(settings.STANFORD_POS_TAGGER,  str(widget_id) + '.tsv'),
             '-tagSeparator', '_',
             '-props', settings.STANFORD_POS_TAGGER_PROPS])


def stanford_pos_tagger(input_dict, widget):
    name= 'StanfordPosTagger'
    if not input_dict['training_corpus']:
        tagger = StanfordTagger(model_filename=settings.STANFORD_POS_TAGGER_MODEL, path_to_jar=settings.STANFORD_POS_TAGGER_JAR, java_options='-mx4000m')
    else:
        chunk = input_dict['training_corpus']['chunk']
        corpus = input_dict['training_corpus']['corpus']
        training_corpus=corpus_reader(corpus, chunk)
        tagger = StanfordTagger(model_filename=settings.STANFORD_POS_TAGGER_MODEL, path_to_jar=settings.STANFORD_POS_TAGGER_JAR, java_options='-mx4000m')
        tagger.train(list(training_corpus), widget.id)
        tagger = StanfordTagger(model_filename=os.path.join(settings.STANFORD_POS_TAGGER,  str(widget.id) + '.tagger'), path_to_jar=settings.STANFORD_POS_TAGGER_JAR, java_options='-mx4000m')
    return {'pos_tagger': {
                'function':'tag_sents',
                'object': tagger,
                'name': name,

            }
    }


def tree_tagger(input_dict, widget):
    name= 'TreeTagger'
    if not input_dict['training_corpus']:
        tagger = TreeTagger(language='english', widget_id=widget.id)
    else:
        chunk = input_dict['training_corpus']['chunk']
        corpus = input_dict['training_corpus']['corpus']
        training_corpus=corpus_reader(corpus, chunk)
        tagger = TreeTagger(language='english', widget_id=widget.id, trained=True)
        tagger.train(list(training_corpus))
    return {'pos_tagger': {
                'function':'tag',
                'object': tagger,
                'name': name,
            }
    }


def crf_pos_tagger(input_dict, widget):
    name= 'CRFPosTagger'
   
    crf_tagger = CRFTagger()
    chunk = input_dict['training_corpus']['chunk']
    corpus = input_dict['training_corpus']['corpus']
    training_corpus=corpus_reader(corpus, chunk)
    model_path = os.path.join(settings.CRF_TAGGER,  str(widget.id) + '.crf.tagger')
    crf_tagger.train(list(training_corpus), model_path)
    tagger = PickleableCRFTagger(model_path)
    #print(crf_tagger.tag_sents([['test', 'is'], ['hey', 'there']]))

    return {'pos_tagger': {
                'function':'tag_sents',
                'object': tagger,
                'name': name,
            }
    }

def tnt_pos_tagger(input_dict):
    name= 'TNTPosTagger'

    tnt_tagger = tnt.TnT()
    chunk = input_dict['training_corpus']['chunk']
    corpus = input_dict['training_corpus']['corpus']
    training_corpus=corpus_reader(corpus, chunk)
    tnt_tagger.train(list(training_corpus))

    return {'pos_tagger': {
                'function':'tagdata',
                'object': tnt_tagger,
                'name': name
            }
    }

def nlp4j_tagger(input_dict, widget):
    name= 'NLP4jPosTagger'
    if not input_dict['training_corpus']:
        NLP4J_INPUT = os.path.join(settings.NLP4J_INPUT, str(widget.id) + '_nlp4j_input.txt')
        NLP4J_CONFIG = os.path.join(settings.NLP4J_CONFIG, 'config_pretrained.xml')
        NLP4J_TRAIN_SET = os.path.join(settings.NLP4J_INPUT, str(widget.id) + '_nlp4j_train.tsv')
        NLP4J_DEV_SET = os.path.join(settings.NLP4J_INPUT, str(widget.id) + '_nlp4j_dev.tsv')
        NLP4J_MODEL = os.path.join(settings.NLP4J_MODEL, str(widget.id) + '.xz')
        NLP4J_CONFIG_TRAINED = os.path.join(settings.NLP4J_CONFIG, "config_trained.xml")
        nlp4j_tagger = nlp4j.NLP4JTagger(settings.NLP4J_BINARIES, NLP4J_CONFIG, NLP4J_INPUT, NLP4J_TRAIN_SET, NLP4J_DEV_SET, NLP4J_MODEL, NLP4J_CONFIG_TRAINED, pretrained=False)
    else:
        chunk = input_dict['training_corpus']['chunk']
        corpus = input_dict['training_corpus']['corpus']
        training_corpus=corpus_reader(corpus, chunk)

        NLP4J_INPUT = os.path.join(settings.NLP4J_INPUT, str(widget.id) + '_nlp4j_input.txt')
        NLP4J_MODEL = os.path.join(settings.NLP4J_MODEL, str(widget.id) + '.xz')
        NLP4J_CONFIG = os.path.join(settings.NLP4J_CONFIG, str(widget.id) + '.xml')
        NLP4J_CONFIG_TRAINED = os.path.join(settings.NLP4J_CONFIG, "config_trained.xml")
        NLP4J_TRAIN_SET = os.path.join(settings.NLP4J_INPUT, str(widget.id) + '_nlp4j_train.tsv')
        NLP4J_DEV_SET = os.path.join(settings.NLP4J_INPUT, str(widget.id) + '_nlp4j_dev.tsv')
        nlp4j_tagger = nlp4j.NLP4JTagger(settings.NLP4J_BINARIES, NLP4J_CONFIG, NLP4J_INPUT, NLP4J_TRAIN_SET, NLP4J_DEV_SET, NLP4J_MODEL, NLP4J_CONFIG_TRAINED, pretrained=False)
        nlp4j_tagger.train(list(training_corpus))

    return {'pos_tagger': {
            'function':'decode',
            'object':nlp4j_tagger,
            'name': name,
        }
    }












import subprocess
import os
import xml.etree.cElementTree as ET
import csv

interpelations = {'NP':'NNP', 'NPS':'NNPS', 'PP':'PRP', 'PP$':'PRP$', 
                  'VD':'VB', 'VDD':'VBD', 'VDG':'VBG', 'VDN':'VBN', 'VDZ': 'VBZ', 'VDP':'VBP',
                  'VH':'VB', 'VHD':'VBD', 'VHG':'VBG', 'VHN':'VBN', 'VHZ': 'VBZ', 'VHP':'VBP',
                  'VV':'VB', 'VVD':'VBD', 'VVG':'VBG', 'VVN':'VBN', 'VVZ': 'VBZ', 'VVP':'VBP' }


class NLP4JTagger():

    def __init__(self, path_to_bin, config_file, input_path, train_path, dev_path, model_path, train_config_file, pretrained=True):
        self.path_to_bin = path_to_bin
        self.config_file = config_file
        self.train_path = train_path
        self.dev_path = dev_path
        self.input_path = input_path
        self.model_path = model_path
        self.train_config_file = train_config_file

    def train(self, sentences, mode="pos"):

        if os.path.exists(self.train_path):
            os.remove(self.train_path)
      
        with open(self.train_path, "wb") as csv_file:
            writer = csv.writer(csv_file, delimiter='\t')
            for sentence in sentences:
                for word, tag in sentence:
                     writer.writerow((word, tag))

        if os.path.exists(self.dev_path):
            os.remove(self.dev_path)

        with open(self.dev_path, "wb") as csv_file:
            writer = csv.writer(csv_file, delimiter='\t')
            for sentence in sentences:
                for word, tag in sentence:
                     writer.writerow((word, tag))

        self.generate_xml()

        args =[]
        args.append(os.path.join(self.path_to_bin, 'nlptrain'))
        args.extend(["-c", self.train_config_file])
        args.extend(["-t", self.train_path])
        args.extend(["-mode", mode])
        args.extend(["-m", self.model_path])
        args.extend(["-d", self.dev_path])
       
        print(args)
        popen = subprocess.Popen(args, stdout=subprocess.PIPE, bufsize=1)
        for line in iter(popen.stdout.readline, ""):
            print(line)
        popen.communicate()


    def decode(self, sentences):
        input_ext="*" 
        utput_ext="nlp" 
        format="raw"
        threads=2

        old = sentences

        if isinstance(sentences, list):
            if os.path.exists(self.input_path):
                os.remove(self.input_path)
            f = open(self.input_path, 'a')
            for sentence in sentences:
                #sentence = ' '.join(sentence)
                #sentence = sentence.replace(" 's ", "'s ")
                #sentence = sentence.replace(" 'm ", "'m ")
                #sentence = sentence.replace(" 're ", "'re ")
                #sentence = sentence.replace(" 'll ", "'ll ")
                #sentence = sentence.replace(" 've ", "'ve ")
                #sentence = sentence.replace(" 'd ", "'d ")
                #sentence = sentence.replace('\xe2\x84\x87"', "\xe2\x84\x87")
                #sentence = re.sub(r" ([^ ']+)' ", r" \1 ", sentence)
                #sentence = re.sub(r" '([^ ']+) ", r" \1 ", sentence)
                #sentence = re.sub(r" ([^ ']+)-- ", r" \1 ", sentence)
                #sentence = re.sub(r" ([^ .]+)\. ", r" \1 ", sentence)
                for token in sentence:
                    if len(token) == 0:
                        f.write(' -NONE- ')
                    elif token == '###':
                        f.write(' # ')
                    else:
                        f.write(" " + token + " ")
                f.write(' ### ') 
            f.close()
           
        args = []
        args.append(os.path.join(self.path_to_bin, 'nlpdecode'))
        args.extend(["-c", self.config_file])
        args.extend(["-i", self.input_path])
        popen = subprocess.Popen(args, stdout=subprocess.PIPE, bufsize=1)
        popen.communicate()

        f = open(self.input_path + '.nlp', 'r')
        taggedSnt = f.read()
        f.close()

        
        os.remove(self.input_path)
        os.remove(self.input_path + '.nlp')

        sentences = []
        sentence = []
        splitted_tags = taggedSnt.split('\n')[:-1]
        last = len(splitted_tags) - 1
        for i, line in enumerate(splitted_tags):
            line = line.strip()
            try:
                word = line.split('\t')[1].strip()
                tag = line.split('\t') [3].strip()

                if word == '###':
                    sentences.append(sentence)
                    sentence = []
                else:
                    if tag == 'SENT':
                        sentence.append((word, '.'))
                    else:
                        if tag in interpelations:
                            sentence.append((word, interpelations[tag]))
                        else:
                            sentence.append((word, tag))
            except:
                pass
        
        

        checked_sentences = []
        for i, sentence in enumerate(sentences):
            if len(old[i]) != len(sentence):
                sentence = self.fixTokens(old[i], sentence)
            checked_sentences.append(sentence)
        return checked_sentences


    def fixTokens(self, standard, new):
        standard = [u"#" if t==u"###" else t for t in standard]
        fixed = []
        fix = 0
        new_tag = ""
        messedup_sent = False
        if len(standard) < len(new) or messedup_sent:
            new_token=""
            for i, token in enumerate(new):
                word = token[0]
                tag = token[1]
                if word == standard[fix] and new_token=="":
                    fix += 1
                    fixed.append(token)
                else:
                    revert_token = new_token
                    old_token = new_token
                    new_token += word
                    if not standard[fix].startswith(new_token):
                        old_token += " " + word
                        new_token = old_token
                    if not standard[fix].startswith(new_token):
                        #the most complicated scenario with completely wrong tokens
                        messedup_sent = True
                        for j in range(i, len(new)):
                            fixed.append(new[j])
                        break
                    else:
                        if len(new_tag) == 0 or len(word) > 1:
                            new_tag = tag
                        if new_token == standard[fix]:
                            fixed.append((new_token, tag))
                            fix += 1
                            new_token = ""
                            new_tag = ""
            #if not len(fixed) == len(standard):
            #    print('ni popravu: ', standard, new, fixed)

            new = fixed
        fixed = []
        fix = 0
        new_tag = ""
        if len(standard) > len(new) or messedup_sent:
            sliced = ""
            for i, token in enumerate(standard):
                word = token
                predicted_token = new[fix][0]
                if word == predicted_token:
                    fixed.append((token, new[fix][1]))
                    fix += 1
                else:
                    if sliced == "":
                        sliced = predicted_token
                    fixed.append((word, new[fix][1]))
                    sliced = sliced[len(word):]
                    if sliced == "":
                        fix += 1
                    
            #if not len(fixed) == len(standard):
            #    print('ni popravu dolzga: ', standard, fixed)
            #else:
            #    print('popravu', standard, fixed)
            new = fixed

        if not len(new) == len(standard):
            print('ni popravu: ', standard, new)

        return new

    def generate_xml(self):
        root = ET.Element("configuration")
        tsv = ET.SubElement(root, "tsv")
        lexica =  ET.SubElement(root, "lexica")
        models = ET.SubElement(root, "models")

        ET.SubElement(tsv, "column", index="1", field="form")
        ET.SubElement(lexica, "ambiguity_classes", field="word_form_simplified_lowercase").text = "edu/emory/mathcs/nlp/lexica/en-ambiguity-classes-simplified-lowercase.xz"
        ET.SubElement(lexica, "word_clusters", field="word_form_simplified_lowercase").text = "edu/emory/mathcs/nlp/lexica/en-brown-clusters-simplified-lowercase.xz"
        ET.SubElement(models, "pos").text = self.model_path

        
        tree = ET.ElementTree(root)
        tree.write(self.config_file)


# -*- coding: utf-8 -*-
# Natural Language Toolkit: Interface to the TreeTagger POS-tagger
#
# Copyright (C) Mirko Otto
# Author: Mirko Otto <dropsy@gmail.com>

"""
A Python module for interfacing with the Treetagger by Helmut Schmid.
"""

import os
import subprocess
from subprocess import Popen, PIPE
from django.conf import settings
import re

from nltk.internals import find_binary, find_file
from nltk.tag.api import TaggerI
from collections import defaultdict
from django.conf import settings

def tUoB(obj, encoding='utf-8'):
    if isinstance(obj, basestring):
        if not isinstance(obj, unicode):
            obj = unicode(obj, encoding)
    return obj

_treetagger_url = 'http://www.cis.uni-muenchen.de/~schmid/tools/TreeTagger/'

_treetagger_languages = {
u'latin-1':['latin', 'latinIT', 'mongolian', 'swahili'],
u'utf-8' : ['bulgarian', 'dutch', 'english', 'estonian', 'finnish', 'french', 'galician', 'german', 'italian', 'polish', 'russian', 'slovak', 'slovak2', 'spanish']}

"""The default encoding used by TreeTagger: utf-8. u'' means latin-1; ISO-8859-1"""
_treetagger_charset = [u'utf-8', u'latin-1']

interpelations = {'NP':'NNP', 'NPS':'NNPS', 'PP':'PRP', 'PP$':'PRP$', 
                  'VD':'VB', 'VDD':'VBD', 'VDG':'VBG', 'VDN':'VBN', 'VDZ': 'VBZ', 'VDP':'VBP',
                  'VH':'VB', 'VHD':'VBD', 'VHG':'VBG', 'VHN':'VBN', 'VHZ': 'VBZ', 'VHP':'VBP',
                  'VV':'VB', 'VVD':'VBD', 'VVG':'VBG', 'VVN':'VBN', 'VVZ': 'VBZ', 'VVP':'VBP', 'IN/that':'IN' }
                   

class TreeTagger(TaggerI):
    ur"""
    A class for pos tagging with TreeTagger. The input is the paths to:
     - a language trained on training data
     - (optionally) the path to the TreeTagger binary
     - (optionally) the encoding of the training data (default: utf-8)

    This class communicates with the TreeTagger binary via pipes.

    Example:

    .. doctest::
        :options: +SKIP

        >>> from treetagger import TreeTagger
        >>> tt = TreeTagger(encoding='utf-8',language='english')
        >>> tt.tag(u'What is the airspeed of an unladen swallow ?')
        [[u'What', u'WP', u'What'],
         [u'is', u'VBZ', u'be'],
         [u'the', u'DT', u'the'],
         [u'airspeed', u'NN', u'airspeed'],
         [u'of', u'IN', u'of'],
         [u'an', u'DT', u'an'],
         [u'unladen', u'JJ', u'<unknown>'],
         [u'swallow', u'NN', u'swallow'],
         [u'?', u'SENT', u'?']]

    .. doctest::
        :options: +SKIP

        >>> from treetagger import TreeTagger
        >>> tt = TreeTagger()
        >>> tt.tag(u'Das Haus ist sehr schön und groß. Es hat auch einen hübschen Garten.')
        [[u'Das', u'ART', u'd'],
         [u'Haus', u'NN', u'Haus'],
         [u'ist', u'VAFIN', u'sein'],
         [u'sehr', u'ADV', u'sehr'],
         [u'sch\xf6n', u'ADJD', u'sch\xf6n'],
         [u'und', u'KON', u'und'],
         [u'gro\xdf', u'ADJD', u'gro\xdf'],
         [u'.', u'$.', u'.'],
         [u'Es', u'PPER', u'es'],
         [u'hat', u'VAFIN', u'haben'],
         [u'auch', u'ADV', u'auch'],
         [u'einen', u'ART', u'ein'],
         [u'h\xfcbschen', u'ADJA', u'h\xfcbsch'],
         [u'Garten', u'NN', u'Garten'],
         [u'.', u'$.', u'.']]
    """

    def __init__(self, path_to_home=None, language='english', 
                 encoding='utf-8', verbose=False, abbreviation_list=None, widget_id=None, trained=False):
        """
        Initialize the TreeTagger.

        :param path_to_home: The TreeTagger binary.
        :param language: Default language is german.
        :param encoding: The encoding used by the model. Unicode tokens
            passed to the tag() and batch_tag() methods are converted to
            this charset when they are sent to TreeTagger.
            The default is utf-8.

            This parameter is ignored for str tokens, which are sent as-is.
            The caller must ensure that tokens are encoded in the right charset.
        """
        tagger_bin = os.path.join(settings.TREE_TAGGER, 'bin')
        treetagger_paths = [tagger_bin]
        treetagger_paths = map(os.path.expanduser, treetagger_paths)
        self._abbr_list = abbreviation_list
        self.widget_id = widget_id
        self.params = os.path.join(settings.TREE_TAGGER, 'lib')
        self.trained = trained

        try:
            self._encoding = encoding
            if language=='english':
                treetagger_bin_name = 'tree-tagger'
                train_treetagger_bin_name = 'train-tree-tagger'

        except KeyError as e:
                raise LookupError('NLTK was unable to find the TreeTagger bin!')

        self._treetagger_bin = find_binary(
            treetagger_bin_name, path_to_home,
            env_vars=('TREETAGGER', 'TREETAGGER_HOME'),
            searchpath=treetagger_paths,
            url=_treetagger_url,
            verbose=verbose)

        self._train_treetagger_bin = find_binary(
            train_treetagger_bin_name, path_to_home,
            env_vars=('TREETAGGER', 'TREETAGGER_HOME'),
            searchpath=treetagger_paths,
            url=_treetagger_url,
            verbose=verbose)


    def train(self, sentences):
        lexicon = defaultdict(list)
        train_path = os.path.join(settings.TREE_TAGGER, str(self.widget_id) + ".train")
        if os.path.exists(train_path):
            os.remove(train_path)
        f = open(train_path, 'a')
        for sent in sentences:
            for i, token in enumerate(sent):
                word, tag = token
                if len(word) == 0:
                    word = '-NONE-'
                    tag = '-NONE-'
                if len(tag) == 0:
                    tag = '-NONE-'
                if i == len(sent) - 1 and tag == '.':
                    tag = 'SENT'
                f.write(word + '\t' + tag + '\n')
                if tag not in lexicon[word]:
                    lexicon[word].append(tag)
        f.close()
        lexicon_path = os.path.join(settings.TREE_TAGGER, str(self.widget_id) + ".lex")
        if os.path.exists(lexicon_path):
            os.remove(lexicon_path)
        f = open(lexicon_path, 'a')
        cd = False
        print(lexicon['!'])
        for key in sorted(lexicon):
            tag_list = ""
            for i, tag in enumerate(lexicon[key]):
                if tag == 'CD':
                    if not cd:
                        if i == len(lexicon[key]) - 1:
                            tag_list += '\t' + tag + ' ' + '-\n'
                        else:
                            tag_list += '\t' + tag + ' ' + '-'
                        cd = True
                    else:
                        if tag_list and i == len(lexicon[key]) - 1:
                            tag_list += '\n'

                else:
                    if i == len(lexicon[key]) - 1:
                        tag_list += '\t' + tag + ' ' + '-\n'
                    else:
                        tag_list += '\t' + tag + ' ' + '-'
            if tag_list:
                f.write(key.encode('utf8')  + tag_list.encode('utf8'))
        f.close()
        unknown_words = os.path.join(settings.TREE_TAGGER, "tree_tagger_unknown_words.txt")

        param_path = os.path.join(self.params, str(self.widget_id) + '.par')
        if os.path.exists(param_path):
            os.remove(param_path)

        p = Popen([self._train_treetagger_bin, lexicon_path, unknown_words, train_path, param_path], stdin=PIPE, stdout=PIPE, stderr=PIPE)
        a = p.communicate()
        print(a)


    def tag(self, sentences):
        """Tags a single sentence: a list of words.
        The tokens should not contain any newline characters.
        """
        encoding = self._encoding
        old = sentences
        # Write the actual sentences to the temporary input file
        if not self.trained:
            param_path = os.path.join(self.params, 'english-utf8.par')
        else:
            param_path = os.path.join(self.params, str(self.widget_id) + '.par')

        if isinstance(sentences, list):
            tokens = []
            new_sents = []
            input_path = os.path.join(settings.TREE_TAGGER, str(self.widget_id) + ".txt")
            if os.path.exists(input_path):
                os.remove(input_path)
            f = open(os.path.join(settings.TREE_TAGGER, str(self.widget_id) + ".txt"), 'a')

            for sentence in sentences:
                #sentence = ' '.join(sentence)
                #sentence = sentence.replace(" 's ", "'s ")
                #sentence = sentence.replace(" 'm ", "'m ")
                #sentence = sentence.replace(" 're ", "'re ")
                #sentence = sentence.replace(" 'll ", "'ll ")
                #sentence = sentence.replace(" 've ", "'ve ")
                #sentence = sentence.replace(" 'd ", "'d ")
                #sentence = sentence.replace('\xe2\x84\x87"', "\xe2\x84\x87")
                #sentence = re.sub(r" ([^ ']+)' ", r" \1 ", sentence)
                #sentence = re.sub(r" '([^ ']+) ", r" \1 ", sentence)
                #sentence = re.sub(r" ([^ ']+)-- ", r" \1 ", sentence)
                #sentence = re.sub(r" ([^ .]+)\. ", r" \1 ", sentence)
                for token in sentence:
                    if len(token) == 0:
                        f.write('-NONE-\n')
                        tokens.append('-NONE-')
                    elif token == '###':
                        f.write('#\n')
                        tokens.append('#')
                    else:
                        f.write(token + '\n')
                        tokens.append(token)
                f.write('###\n')
                tokens.append('###')
               
                new_sents.append(sentence)
            f.close()
               
            p = Popen([self._treetagger_bin, param_path, os.path.join(settings.TREE_TAGGER, str(self.widget_id) + ".txt")], stdin=PIPE, stdout=PIPE, stderr=PIPE)
                
            (taggedSnt, stderr) = p.communicate()
            
            sentences = []
            sentence = []
            splitted_tags = taggedSnt.split('\n')[:-1]
            
            for word, tag in zip(tokens, splitted_tags):
                try:
                    word = word.strip()
                    tag = tag.strip()
                    if word == '###':
                        sentences.append(sentence)
                        sentence = []
                    else:
                        if tag == 'SENT':
                            sentence.append((word, '.'))
                        else:
                            if tag in interpelations:
                                sentence.append((word, interpelations[tag]))
                            else:
                                sentence.append((word, tag))
                except:
                    pass

        checked_sentences = []
        counter = 0
        for i, sentence in enumerate(sentences):
            if len(old[i]) != len(sentence):
                counter += 1
                sentence = self.fixTokens(old[i], sentence)
            checked_sentences.append(sentence)
        return checked_sentences

    
    
    def fixTokens(self, standard, new):
        standard = [u"#" if t==u"###" else t for t in standard]
        fixed = []
        fix = 0
        new_tag = ""
        messedup_sent = False
        if len(standard) < len(new) or messedup_sent:
            new_token=""
            for i, token in enumerate(new):
                word = token[0]
                tag = token[1]
                if word == standard[fix] and new_token=="":
                    fix += 1
                    fixed.append(token)
                else:
                    revert_token = new_token
                    old_token = new_token
                    new_token += word
                    if not standard[fix].startswith(new_token):
                        old_token += " " + word
                        new_token = old_token
                    if not standard[fix].startswith(new_token):
                        #the most complicated scenario with completely wrong tokens
                        messedup_sent = True
                        for j in range(i, len(new)):
                            fixed.append(new[j])
                        break
                    else:
                        if len(new_tag) == 0 or len(word) > 1:
                            new_tag = tag
                        if new_token == standard[fix]:
                            fixed.append((new_token, tag))
                            fix += 1
                            new_token = ""
                            new_tag = ""
            #if not len(fixed) == len(standard):
            #    print('ni popravu: ', standard, new, fixed)

            new = fixed
        fixed = []
        fix = 0
        new_tag = ""
        if len(standard) > len(new) or messedup_sent:
            sliced = ""
            for i, token in enumerate(standard):
                word = token
                predicted_token = new[fix][0]
                if word == predicted_token:
                    fixed.append((token, new[fix][1]))
                    fix += 1
                else:
                    if sliced == "":
                        sliced = predicted_token
                    fixed.append((word, new[fix][1]))
                    sliced = sliced[len(word):]
                    if sliced == "":
                        fix += 1
                    
            #if not len(fixed) == len(standard):
            #    print('ni popravu dolzga: ', standard, fixed)
            #else:
            #    print('popravu', standard, fixed)
            new = fixed

        if not len(new) == len(standard):
            print('ni popravu: ', standard, new)

        return new





    






import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='tf_taggers',
    version='0.0.1',
    packages=['tf_taggers'],
    include_package_data=True,
    license='MIT License',
    description='TextFlows taggers module',
    long_description=README,
    url='https://github.com/xflows/tf_taggers',
    author='Matej Martinc',
    author_email='matej.martinc@ijs.si',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        "nltk==3.2.1",
        "django-discover-runner==1.0",
        "numpy",
        "scipy",
        "scikit-learn==0.16.1",
        "docx",
        "requests",
        "pdfminer",
        "python-crfsuite"
    ]
)
